@@ -4,7 +4,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = 0.2002;
+our $VERSION = 0.21;
 
 =head1 NAME
 
@@ -120,6 +120,15 @@ B<The response> may contain regular keys, typically alphanumeric,
 as well as a predefined set of dash-prefixed keys to control
 Neaf itself.
 
+    return {
+        -view     => 'TT',
+        -template => 'users.html',
+        users     => \@list,
+        extras    => \%hash,
+    };
+
+And that's it.
+
 I<-Note -that -dash-prefixed -options -look -antique
 even to the author of this writing.
 However, it is a concise and B<visible> way to separate
@@ -150,6 +159,9 @@ Otherwise it's ignored for security reasons.
 =item * -location - HTTP Location: header for 3xx statuses.
 This is set by C<$request-E<gt>redirect(...)>.
 
+B<DEPRECATED.> This will be phased out at some point,
+use C<-header => [ location =E<gt> ... ]> instead.
+
 =item * -serial - if present, the JS view will render this instead of
 the whole response hash.
 This can be used e.g. to return non-hash data in REST API.
@@ -162,7 +174,7 @@ Default is 200 if the app managed to live through, and 500 if it died.
 
 =item * -type - Content-type HTTP header.
 View module may set this parameter if unset.
-Default: C<"text/html">.
+Default is generated by the renderer - see L<MVC::Neaf::View>.
 
 =item * -view - select B<View> module.
 Views are initialized lazily and cached by the framework.
@@ -173,7 +185,7 @@ C<MVC::Neaf-E<gt>load_view( "name" =E<gt> $your_view );> (see below).
 
 The default is the JS aka L<MVC::Neaf::View::JS> engine.
 Adding C<-template> key will cause switching to C<MVC::Neaf::View::TT>,
-but it is deprecated and will go away in v.0.20.
+but it is deprecated and will go away in v.0.25.
 
 =back
 
@@ -182,32 +194,51 @@ and will be passed to the View module as of current,
 they are not guaranteed to work in the future.
 Please either avoid them, or send patches.
 
-=head1 MAIN API
+=head1 FUNCTIONAL AND OBJECT-ORIENTED API
 
-These methods are generally called during the setup phase of the application.
+A C<:sugar> keyword must be added to the C<use> statement
+to get access to the functional API.
+The need to do so MAY be removed in the future.
 
-Although L<MVC::Neaf> can be instantiated more than once,
-having many of them inside one Perl interpreter is of little use yet.
+All prototyped declarative functions described below
+are really frontends to an L<MVC::Neaf> object
+that accumulates the knowledge about your application.
 
-For brevity, a prototyped C<neaf()> function invokes all kind of methods on
-the default instance, or just returns it if called without arguments.
+Though more than one such objects can be created,
+for now there is little use in doing so.
+
+Given the above, functional and object-oriented ways
+to declare the same thing will now follow in pairs.
+
+Returned value, if unspecified, is always the Neaf object
+(but who cares).
+
+=head2 neaf;
+
+Without arguments, returns the default Neaf instance
+that handles all the functional calls.
+See complete description below.
 
 =cut
 
 use Carp;
 use Encode;
+use HTTP::Headers::Fast;
+use MIME::Base64;
 use Module::Load;
 use Scalar::Util qw(blessed looks_like_number);
 use URI::Escape;
 use parent qw(Exporter);
 
-our @EXPORT_OK = qw( neaf_err neaf get post head put );
+our @EXPORT_OK = qw( neaf_err );
+my  @EXPORT_SUGAR = qw( neaf ); # Will populate later - see @ALL_METHODS below
 our %EXPORT_TAGS = (
-    sugar => [qw[ neaf get post head put ]],
+    sugar => \@EXPORT_SUGAR,
 );
 
 use MVC::Neaf::Util qw(http_date canonize_path path_prefixes run_all run_all_nodie);
 use MVC::Neaf::Request::PSGI;
+use MVC::Neaf::Exception;
 
 our $Inst;
 
@@ -217,15 +248,35 @@ my %FORM_ENGINE = (
     wildcard => 'MVC::Neaf::X::Form::Wildcard',
 );
 
-=head2 get '/path' => sub { ... }, %options
+=head2 get + post '/path' => CODEREF, %options
 
-=head2 post '/path' => sub { ... }, %options
+The following aliases exist to create routes:
 
-=head2 get+post '/path' => sub { ... }, %options
+=over
 
-=head2 neaf route => '/path' => sub { ... }, %options
+=item * get
 
-This needs a C<method =E<gt> [ ... ]> or it's just a 'GET + POST' by default.
+=item * post
+
+=item * head
+
+=item * put
+
+=item * patch
+
+=item * del for C<DELETE>
+
+=back
+
+These may be combined using C<+> sign, or alternatively there's
+
+=over
+
+=item * any [ 'get', 'post', 'foobar' ]
+
+=back
+
+to handle arbitrary request method sets.
 
 =head2 $neaf->route( '/path' => CODEREF, %options )
 
@@ -236,7 +287,8 @@ will now be directed to CODEREF.
 
 Longer paths are GUARANTEED to be checked first.
 
-Dies if the same method and path combination is given again.
+Dies if the same method and path combination is given again
+(but see C<tentative> and C<override> below).
 Multiple methods may be given for the same path.
 
 Exactly one leading slash will be prepended no matter what you do.
@@ -284,7 +336,7 @@ Must be an object with a C<render> method, or a CODEREF
 receiving hashref and returning a list of two scalars
 (content and content-type).
 
-B<DEPRECATED>. Use -view instead, semantics are the same.
+B<DEPRECATED>. Use -view instead, meaning is exactly the same.
 
 =item * cache_ttl - if set, set Expires: HTTP header accordingly.
 
@@ -292,12 +344,31 @@ B<EXPERIMENTAL>. Name and semantics MAY change in the future.
 
 =item * default - a C<\%hash> of values that will be added to results
 EVERY time the handler returns.
-Consider using C<neaf default ...> below if you need these added
-to multiple handlers.
+Consider using C<neaf default ...> below if you need to add
+the same values to multiple handlers.
+
+=item * override => 1 - replace old route even if it exists.
+If not set, route collisions causes exception.
+Use this if you know better.
+
+This still warns.
+
+B<EXPERIMENTAL.> Name and meaning may change in the future.
+
+=item * tentative => 1 - don't complain if this route is replaced.
+
+E.g. if setting a static stub for method to be added later.
+
+B<EXPERIMENTAL.> Name and meaning may change in the future.
 
 =item * description - just for information, has no action on execution.
 This will be displayed if application called with --list (see L<MVC::Neaf::CLI>).
 
+=item * public => 0|1 - a flag just for information.
+In theory, public endpoints should be searchable from the outside
+while non-public ones should only be reachable from other parts of application.
+This is not enforced whatsoever.
+
 =back
 
 Also, any number of dash-prefixed keys MAY be present.
@@ -314,9 +385,11 @@ it v.0.25.
 
 my $year = 365 * 24 * 60 * 60;
 my %known_route_args;
-$known_route_args{$_}++ for qw(default method view cache_ttl
+$known_route_args{$_}++ for qw(
+    default method view cache_ttl
     path_info_regex param_regex
-    description caller);
+    description caller tentative override public
+);
 
 sub route {
     my $self = shift;
@@ -337,6 +410,9 @@ sub route {
     my (%args) = @_;
     $self = $Inst unless ref $self;
 
+    $self->_croak( "handler must be a coderef, not ".ref $sub )
+        unless UNIVERSAL::isa( $sub, "CODE" );
+
     # check defaults to be a hash before accessing them
     $self->_croak( "default must be unblessed hash" )
         if $args{default} and ref $args{default} ne 'HASH';
@@ -350,19 +426,21 @@ sub route {
     $self->_croak( "Unexpected keys in route setup: @extra" )
         if @extra;
 
-    $path = canonize_path( $path );
+    $args{path} = $path = canonize_path( $path );
 
     _listify( \$args{method}, qw( GET POST ) );
+    $_ = uc $_ for @{ $args{method} };
 
-    my @dupe = grep { exists $self->{route}{$path}{$_} } @{ $args{method} };
-    $self->_croak( "Attempting to set duplicate handler for [@dupe] "
-        .( length $path ? $path : "/" ) )
-            if @dupe;
+    $self->_croak("Public endpoint must have nonempty description")
+        if $args{public} and not $args{description};
+
+    $self->_detect_duplicate( \%args );
 
     # Do the work
     my %profile;
-    $profile{code}     = $sub;
-    $profile{caller}   = $args{caller} || [caller(0)]; # file,line
+    $profile{code}      = $sub;
+    $profile{tentative} = $args{tentative};
+    $profile{override}  = $args{override};
 
     # Always have regex defined to simplify routing
     $profile{path_info_regex} = (defined $args{path_info_regex})
@@ -372,6 +450,9 @@ sub route {
     # Just for information
     $profile{path}        = $path;
     $profile{description} = $args{description};
+    $profile{public}      = $args{public} ? 1 : 0;
+    $profile{caller}      = $args{caller} || [caller(0)]; # save file,line
+    $profile{where}       = "at $profile{caller}[1] line $profile{caller}[2]";
 
     if (my $view = $args{view}) {
         carp "NEAF: route(): view argument is deprecated, use -view instead";
@@ -407,6 +488,7 @@ sub route {
 
     # ready, shallow copy handler & burn cache
     delete $self->{route_re};
+
     $self->{route}{ $path }{$_} = { %profile, my_method => $_ }
         for @{ $args{method} };
 
@@ -424,22 +506,67 @@ sub _dup_route {
     $profile ||= $self->{last_added};
     my $path = $profile->{path};
 
-    $self->_croak( "Attempting to set duplicate handler for [$method] "
-        .($path || '/'))
-            if $self->{route}{ $path }{$method};
-
-    $self->{route}{ $path }{$method}
-        and $self->_croak("Attempting to set duplicate ");
+    $self->_detect_duplicate($profile);
 
     delete $self->{route_re};
     $self->{route}{ $path }{$method} = { %$profile, my_method => $method };
 };
 
+# in: { method => [...], path => '/...', tentative => 0|1, override=> 0|1 }
+# out: none
+# spoils $method if tentative
+# dies/warns if violations found
+sub _detect_duplicate {
+    my ($self, $profile) = @_;
+
+    my $path = $profile->{path};
+    # Handle duplicate route definitions
+    my @dupe = grep {
+        exists $self->{route}{$path}{$_}
+        and !$self->{route}{$path}{$_}{tentative};
+    } @{ $profile->{method} };
+
+    if (@dupe) {
+        my %olddef;
+        foreach (@dupe) {
+            my $where = $self->{route}{$path}{$_}{where};
+            push @{ $olddef{$where} }, $_;
+        };
+
+        # flatten olddef hash, format list
+        my $oldwhere = join ", ", map { "$_ [@{ $olddef{$_} }]" } keys %olddef;
+        my $oldpath = $path || '/';
+
+        # Alas, must do error message by hand
+        my $caller = [caller 1]->[3];
+        $caller =~ s/.*:://;
+        if ($profile->{override}) {
+            carp( (ref $self)."->$caller: Overriding old handler for"
+                ." $oldpath defined $oldwhere");
+        } elsif( $profile->{tentative} ) {
+            # just skip duplicate methods
+            my %filter;
+            $filter{$_}++ for @{ $profile->{method} };
+            delete $filter{$_} for @dupe;
+            $profile->{method} = [keys %filter];
+        } else {
+            croak( (ref $self)."->$caller: Attempting to set duplicate handler for"
+                ." $oldpath defined $oldwhere");
+        };
+    };
+};
+
 =head2 neaf static => '/path' => ...
 
-=head2 static( $req_path => $file_path, %options )
+=head2 $neaf->static( $req_path => $file_path, %options )
 
 Serve static content located under C<$file_path>.
+Both directories and single files may be added.
+
+If an arrayref of C<[ $content, $content_type ]> is given as second argument,
+serve that content from memory instead.
+
+See L<MVC::Meaf::X::Files> for implementation.
 
 %options may include:
 
@@ -473,8 +600,18 @@ By default a localized TT instance is used.
 
 B<EXPERIMENTAL> Name MAY be changed (dir_view etc).
 
+=item * override - override the route that was here before.
+See C<route> above.
+
+=item * tentative - don't complain if replaced later.
+
 =item * description - comment. The default is "Static content at $dir"
 
+=item * public => 0|1 - a flag just for information.
+In theory, public endpoints should be searchable from the outside
+while non-public ones should only be reachable from other parts of application.
+This is not enforced whatsoever.
+
 =back
 
 The content is really handled by L<MVC::Neaf::X::Files>.
@@ -498,22 +635,36 @@ sub static {
     my ($self, $path, $dir, %options) = @_;
     $self = $Inst unless ref $self;
 
+    $options{caller} ||= [caller 0];
+
+    my %fwd_opt;
+    defined $options{$_} and $fwd_opt{$_} = delete $options{$_}
+        for qw( tentative override caller public );
+
+    if (ref $dir eq 'ARRAY') {
+        my $sub = $self->_static_global->preload( $path => $dir )->one_file_handler;
+        return $self->route( $path => $sub, method => 'GET', %fwd_opt,
+            , description => Carp::shortmess( "Static content from memory" ));
+    };
+
     require MVC::Neaf::X::Files;
     my $xfiles = MVC::Neaf::X::Files->new(
         %options, root => $dir, base_url => $path );
-    return $self->route( $xfiles->make_route );
+    return $self->route( $xfiles->make_route, %fwd_opt );
 };
 
-
 =head2 neaf default => '/path' => \%values
 
-=head2 set_path_defaults ( '/path' => \%values )
+=head2 $neaf->set_path_defaults ( '/path' => \%values )
 
 Use given values as defaults for ANY handler below given path.
 A value of '/' means global.
 
-Longer paths override shorter ones; route-specific defaults override these;
-and anything defined inside handler takes over once again.
+Longer paths override shorter ones;
+route-specific defaults override path-base defaults;
+explicit values returned from handler override all or the above.
+
+    neaf default '/api' => { view => 'JS', version => My::Model->VERSION };
 
 =cut
 
@@ -538,7 +689,7 @@ sub set_path_defaults {
 
 =head2 neaf "phase" => sub { ... }, path => [ ... ], exclude => [ ... ]
 
-=head2 add_hook ( phase => CODEREF, %options )
+=head2 $neaf->add_hook ( phase => CODEREF, %options )
 
 Set hook that will be executed on a given request processing phase.
 
@@ -564,7 +715,7 @@ See C<REQUEST PROCESSING PHASES AND HOOKS> below for detailed
 discussion of each phase.
 
 The CODEREF receives one and only argument - the C<$request> object.
-Return value is ignored.
+Return value is B<ignored>.
 
 Use C<$request>'s C<session>, C<reply>, and C<stash> methods
 for communication between hooks.
@@ -661,14 +812,15 @@ sub _listify {
 
 =head2 neaf alias $newpath => $oldpath
 
-=head2 alias( $newpath => $oldpath )
+=head2 $neaf->alias( $newpath => $oldpath )
 
 Create a new name for already registered route.
 The handler will be executed as is,
 but the hooks and defaults will be re-calculated.
 So be careful.
 
-Returns self.
+B<CAUTION.> As of 0.21, C<alias> does NOT follow tentative/override switches.
+This needs to be fixed in the future.
 
 =cut
 
@@ -682,6 +834,7 @@ sub alias {
     $self->{route}{$old}
         or $self->_croak( "Cannot create alias for unknown route $old" );
 
+    # TODO 0.30 restrict methods, handle tentative/override, detect dupes
     $self->_croak( "Attempting to set duplicate handler for path "
         .( length $new ? $new : "/" ) )
             if $self->{route}{ $new };
@@ -695,10 +848,10 @@ sub alias {
 
 =head2 neaf view => 'name' => 'Driver::Class' => %options;
 
-=head2 load_view( $name, $object || coderef || ($module_name, %options) )
+=head2 $neaf->load_view( $name, $object || coderef || ($module_name, %options) )
 
 Setup view under name $name.
-Subsequent requests with C<-view = $name> wouldbe processed by that view
+Subsequent requests with C<-view = $name> would be processed by that view
 object.
 
 Use C<get_view> to fetch the object itself.
@@ -716,6 +869,8 @@ Short aliases JS, TT, and Dumper may be used for MVC::Neaf::View::*
 
 =back
 
+Returns the view object, NOT the calling Neaf object.
+
 =cut
 
 my %view_alias = (
@@ -754,7 +909,7 @@ sub load_view {
 
 =head2 neaf session => $engine => %options
 
-=head2 set_session_handler( %options )
+=head2 $neaf->set_session_handler( %options )
 
 Set a handler for managing sessions.
 
@@ -836,18 +991,42 @@ sub set_session_handler {
     return $self;
 };
 
-=head2 neaf form => name => $validator
-
 =head2 neaf form => name => \%spec, engine => ...
 
 =head2 add_form( name => $validator )
 
-Create a named form for future data validation.
+Create a named form for future query data validation
+via C<$request-E<gt>form("name")>.
 
-Form is created once and applied to different sets of data,
-returning an object containing collected valid data, errors (if any),
+The C<$validator> is one of:
+
+=over
+
+=item * An object with C<validate> method accepting one C<\%hashref>
+argument (the raw form data).
+
+=item * A CODEREF accepting the same argument.
+
+=back
+
+Whatever is returned by validator is forwarded into the controller.
+
+Neaf comes with a set of predefined validator classes that return
+a convenient object that contains collected valid data, errors (if any),
 and an is_valid flag.
 
+The C<engine> parameter of the functional form has predefined values
+C<Neaf> (the default), C<LIVR>, and C<Wildcard> (all case-insensitive)
+poining towards L<MVC::Neaf::X::Form>, L<MVC::Neaf::X::Form::LIVR>,
+and L<MVC::Neaf::X::Form::Wildcard>, respectively.
+
+You are encouraged to use LIVR
+(See L<Validator::LIVR> and L<LIVR grammar|https://github.com/koorchik/LIVR>)
+for anything except super-basic regexp checks.
+
+If an arbitrary class name is given instead, C<new()> will be called
+on that class with \%spec ref as first parameter.
+
 Consider the following script:
 
     use MVC::Neaf qw(:sugar);
@@ -872,15 +1051,6 @@ And by running this one gets
     bash$ curl http://localhost:5000/check?foo=leet
     {"error":{"foo":"BAD_FORMAT"}}
 
-See more in L<MVC::Neaf::X::Form>.
-
-C<engine> parameter has predefined values C<Neaf> (the default),
-C<LIVR>, and C<Wildcard> (all case-insensitive)
-that will point
-towards built-in validation classes. You are encouraged to use LIVR
-(See L<Validator::LIVR> and L<LIVR grammar|https://github.com/koorchik/LIVR>)
-for anything except super-basic regexp checks.
-
 =cut
 
 sub add_form {
@@ -909,7 +1079,7 @@ sub add_form {
 
 =head2 neaf 403 => sub { ... }
 
-=head2 set_error_handler ( status => CODEREF( $req, %options ) )
+=head2 $neaf->set_error_handler ( status => CODEREF( $req, %options ) )
 
 Set custom error handler.
 
@@ -920,7 +1090,7 @@ The following options will be passed to coderef:
 
 =over
 
-=item * status - status being returned (500 in case of 'view');
+=item * status - status being returned;
 
 =item * caller - file:line where the route was set up;
 This is DEPRECATED and will silently disappear around version 0.25
@@ -935,10 +1105,9 @@ The coderef MUST return an unblessed hash just like a normal controller does.
 In case of exception or unexpected return format
 default JSON-based error will be returned.
 
-=head2 set_error_handler ( status => \%hash )
+Also available as C<set_error_handler( status =E<gt> \%hash )>.
 
-Return a static template as C<{ %options, %hash }>.
-This is a synonym to sub { +{ ... } }.
+This is a synonym to C<sub { +{ status =E<gt> $status,  ... } }>.
 
 =cut
 
@@ -947,7 +1116,7 @@ sub set_error_handler {
     $self = $Inst unless ref $self;
 
     $status =~ /^(?:\d\d\d)$/
-        or $self->_croak( "1st arg must be http status or a const(see docs)");
+        or $self->_croak( "1st arg must be http status");
     if (ref $code eq 'HASH') {
         my $hash = $code;
         $code = sub {
@@ -956,7 +1125,7 @@ sub set_error_handler {
             return { -status => $opt{status}, %opt, %$hash };
         };
     };
-    ref $code eq 'CODE'
+    UNIVERSAL::isa($code, 'CODE')
         or $self->_croak( "2nd arg must be callback or hash");
 
     $self->{error_template}{$status} = $code;
@@ -964,8 +1133,7 @@ sub set_error_handler {
     return $self;
 };
 
-
-=head2 on_error( sub { my ($req, $err) = @_ } )
+=head2 $neaf->on_error( sub { my ($req, $err) = @_ } )
 
 Install custom error handler for dying controller.
 Neaf's own exceptions and C<die \d\d\d> status returns will NOT
@@ -993,9 +1161,132 @@ sub on_error {
     return $self;
 };
 
+=head2 $neaf->load_resources( $file || \*FILE )
+
+Load pseudo-files from a file, like templates or static files.
+
+The format is as follows:
+
+    @@ [TT] main.html
+
+    [% some_tt_template %]
+
+    @@ /favicon.ico format=base64 type=png
+
+    iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAMAAABEpIrGAAAABGdBTUEAAL
+    GPC/xhBQAAAAFzUkdCAK7OHOkAAAAgY0hS<....more encoded lines>
+
+I<This is obviously stolen from L<Mojolicious>,
+in a slightly incompatible way.>
+
+If view is specified in brackets, preload template.
+A missing view is skipped, no error.
+
+Otherwise file is considered a static resource.
+
+Extra options may follow file name:
+
+=over
+
+=item * type = ext | mime/type
+
+=item * format=base64
+
+=back
+
+Unknown options are skipped.
+Unknown format value will cause exception though.
+
+B<EXPERIMENTAL>. This method and exact format of data is being worked on.
+
+=cut
+
+my $INLINE_SPEC = qr/^(?:\[(\w+)\]\s+)?(\S+)((?:\s+\w+=\S+)*)$/;
+sub load_resources {
+    my ($self, $file) = @_;
+
+    my $fd;
+    if (ref $file) {
+        $fd = $file;
+    } else {
+        open $fd, "<", $file
+            or $self->_croak( "Failed to open(r) $file: $!" );
+    };
+
+    local $/;
+    my $content = <$fd>;
+    defined $content
+        or $self->_croak( "Failed to read from $file: $!" );
+
+    my @parts = split /^@@\s+(.*\S)\s*$/m, $content, -1;
+    shift @parts;
+    die "Something went wrong" if @parts % 2;
+
+    my %templates;
+    my %static;
+    while (@parts) {
+        # parse file
+        my $spec = shift @parts;
+        my ($dest, $name, $extra) = ($spec =~ $INLINE_SPEC);
+        my %opt = $extra =~ /(\w+)=(\S+)/g;
+        $name or $self->_croak("Bad resource spec format @@ $spec");
+
+        my $content = shift @parts;
+        if (!$opt{format}) {
+            $content =~ s/^\n+//s;
+            $content =~ s/\s+$//s;
+            $content = Encode::decode_utf8( $content, 1 );
+        } elsif ($opt{format} eq 'base64') {
+            $content = decode_base64( $content );
+        } else {
+            $self->_croak("Unknown format $opt{format} in '@@ $spec' in $file");
+        };
+
+        if ($dest) {
+            # template
+            $self->_croak("Duplicate template '@@ $spec' in $file")
+                if defined $templates{lc $dest}{$name};
+            $templates{$dest}{$name} = $content;
+        } else {
+            # static file
+            $self->_croak("Duplicate static file '@@ $spec' in $file")
+                if $static{$name};
+            $static{$name} = [ $content, $opt{type} ];
+        };
+    };
+
+    # now do the loading
+    foreach my $name( keys %templates ) {
+        my $view = $self->get_view( $name, 1 ) or next;
+        $view->can("preload") or next; # TODO 0.30 warn here?
+        $view->preload( %{ $templates{$name} } );
+    };
+    if( %static ) {
+        require MVC::Neaf::X::Files;
+        my $st = $self->_static_global;
+        $st->preload( %static );
+        foreach( keys %static ) {
+            $self->route( $_ => $st->one_file_handler, method => 'GET'
+                , description => "Static resource from $file" );
+        };
+    };
+
+    return $self;
+};
+
+# TODO 0.30 lame name, find better
+sub _static_global {
+    my $self = shift;
+
+    return $self->{global_static} ||= do {
+        require MVC::Neaf::X::Files;
+        MVC::Neaf::X::Files->new( root => '/dev/null' );
+    };
+};
+
 =head2 neaf->run()
 
-=head2 run()
+=head2 $neaf->run()
 
 Run the applicaton.
 This should be the last statement in your appication main file.
@@ -1004,6 +1295,7 @@ If called in void context, assumes execution as CGI
 and prints results to C<STDOUT>.
 If command line options are present at the moment,
 enters debug mode via L<MVC::Neaf::CLI>.
+Call C<perl yourapp.pl --help> for more.
 
 Otherwise returns a PSGI-compliant coderef.
 This will also happen if you application is C<require>'d,
@@ -1148,9 +1440,9 @@ Current request processing diagram looks as follows:
     . <- pre_cleanup [path traversal long to short] [no effect on headers]
    [*] request destroyed
 
-=head3 pre_route
+=head2 pre_route
 
-Executed AFTER the event has been received, but BEFORE the path has been
+Executed AFTER the request has been received, but BEFORE the path has been
 resolved and handler found.
 
 Dying in this phase stops both further hook processing and controller execution.
@@ -1161,7 +1453,7 @@ Options C<path> and C<exclude> are not available on this stage.
 May be useful for mangling path.
 Use C<$request-E<gt>set_path($new_path)> if you need to.
 
-=head3 pre_logic
+=head2 pre_logic
 
 Executed AFTER finding the correct route, but BEFORE processing the main
 handler code (one that returns C<\%hash>, see C<route> above).
@@ -1176,12 +1468,12 @@ Instead, the corresponding error handler is executed right away.
 B<EXAMPLE>: use this hook to produce a 403 error if the user is not logged in
 and looking for a restricted area of the site:
 
-    MVC::Neaf->set_hook( pre_logic => sub {
+    neaf pre_logic => sub {
         my $request = shift;
         $request->session->{user_id} or die 403;
-    }, path => '/admin', exclude => '/admin/static' );
+    }, path => '/admin', exclude => '/admin/static';
 
-=head3 pre_content
+=head2 pre_content
 
 This hook is run AFTER the main handler has returned or died, but BEFORE
 content rendering/serialization is performed.
@@ -1190,7 +1482,7 @@ C<reply()> hash is available at this stage.
 
 Dying is ignored, only producing a warning.
 
-=head3 pre_render
+=head2 pre_render
 
 This hook is run BEFORE content rendering is performed, and ONLY IF
 the content is going to be rendered,
@@ -1198,7 +1490,7 @@ i.e. no C<-content> key set in response hash on previous stages.
 
 Dying will stop rendering, resulting in a template error instead.
 
-=head3 pre_reply
+=head2 pre_reply
 
 This hook is run AFTER the headers have been generated, but BEFORE the reply is
 actually sent to client. This is the last chance to amend something.
@@ -1209,7 +1501,7 @@ C<reply()> hash is available at this stage.
 
 Dying is ignored, only producing a warning.
 
-=head3 pre_cleanup
+=head2 pre_cleanup
 
 This hook is run AFTER all postponed actions set up in controller
 (via C<-continue> etc), but BEFORE the request object is actually destroyed.
@@ -1223,7 +1515,7 @@ Dying is ignored, only producing a warning.
 
 =cut
 
-=head1 EXPORTED FUNCTIONS
+=head1 HELPER FUNCTIONS
 
 Neaf tries hard to keep user's namespace clean, however,
 some helper functions are needed.
@@ -1257,29 +1549,19 @@ in the controller. E.g.
 
 sub neaf_err(;$) { ## no critic # prototype it for less typing on user's part
     my $err = shift || $@;
-    return unless blessed $err and $err->isa("MVC::Neaf::Exception");
-    die $err;
+    die $err if blessed $err and $err->isa("MVC::Neaf::Exception");
+    die $err if !ref $err and $err =~ /^(\d\d\d)\s/s; # die 403
+    return;
 };
 
-=head2 neaf->...
+=head2 neaf action => @options;
 
-Returns default Neaf instance (C<$MVC::Neaf::Inst>), so that
-C<neaf-E<gt>method_name> is the equivalent of C<MVC::Neaf-E<gt>method_name>.
-
-=head2 neaf shortcut => @options;
-
-Shorter alias to methods described above. Currently supported:
+See the list of available actions:
 
 =over
 
-=item * route - C<route>
-
-=item * error - C<set_error_handler>
-
 =item * view - C<load_view>
 
-=item * hook - C<add_hook>
-
 =item * session - C<set_session_handler>
 
 =item * default - C<set_path_defaults>
@@ -1288,10 +1570,19 @@ Shorter alias to methods described above. Currently supported:
 
 =item * static  - C<static>
 
-=back
+=item * route - C<route>
 
-Also, passing a 3-digit number will trigger C<set_error_handler>,
-and passing a hook phase (see below) will result in setting a hook.
+Don't do this, use C<any> or C<get + post + ...> instead.
+
+=item * hook - C<add_hook>
+
+Don't do this, use phase hame instead.
+
+=item * error - C<set_error_handler>
+
+Don't do this, use 3-digit error code instead.
+
+=back
 
 =cut
 
@@ -1329,6 +1620,10 @@ sub neaf(@) { ## no critic # DSL
         unshift @args, 'engine';
     };
 
+    if ($action eq 'route' ) {
+        carp "neaf route is DEPRECATED, use get+post+put instead";
+    };
+
     my $method = $method_shortcut{$action};
     croak "neaf: don't know how to handle '$action'"
         unless $method and MVC::Neaf->can($method);
@@ -1336,29 +1631,23 @@ sub neaf(@) { ## no critic # DSL
     return $MVC::Neaf::Inst->$method( @args );
 };
 
-=head2 get '/path' => sub { ... }, ...
+# Generate alias subs
+my @ALL_METHODS = qw( get head post put patch delete );
+my %ALIAS;
+$ALIAS{$_} = uc $_ for @ALL_METHODS;
+$ALIAS{del} = delete $ALIAS{delete}; # ouch, no delete '/foo' => bar
+$ALIAS{any} = \@ALL_METHODS;
 
-=head2 head '/path' => sub { ... }, ...
-
-=head2 post '/path' => sub { ... }, ...
-
-=head2 put '/path' => sub { ... }, ...
-
-Set request method handler. c<get + post @args> is equivalent to
-C<get @args>; C<post @args>.
-
-Other methods and combinations (e.g. PATCH) are available via
-
-    neaf route '/path' => sub { ... }, method => [ 'PATCH', 'DELETE' ], ...;
-
-=cut
-
-foreach (qw(get head post put)) {
-    my $method = uc $_;
+foreach (keys %ALIAS) {
+    my $method = $ALIAS{$_};
+    my $is_any = $_ eq 'any';
 
     my $code = sub(@) { ## no critic
-        # get + post sugar
-        if (@_ == 1 and UNIVERSAL::isa( $_[0], __PACKAGE__ )) {
+        # any
+        if ($is_any and ref $_[0] eq 'ARRAY') {
+            $method = shift;
+        } elsif (@_ == 1 and UNIVERSAL::isa( $_[0], __PACKAGE__ )) {
+            # get + post sugar
             return $_[0]->_dup_route( $method );
         };
 
@@ -1369,18 +1658,20 @@ foreach (qw(get head post put)) {
             $path, $handler, @args, method => $method, caller => [caller(0)] );
     };
 
+    push @EXPORT_SUGAR, $_;
     no strict 'refs'; ## no critic
     *{$_} = $code;
 };
+push @EXPORT_OK, @EXPORT_SUGAR;
 
 =head1 DEVELOPMENT AND DEBUGGING METHODS
 
-=head2 run_test( \%PSGI_ENV, %options )
+=head2 $neaf->run_test( \%PSGI_ENV, %options )
 
-=head2 run_test( "/path?param=value", %options )
+=head2 $neaf->run_test( "/path?param=value", %options )
 
 Run a PSGI request and return a list of
-C<($status, HTTP::Headers, $whole_content )>.
+C<($status, HTTP::Headers::Fast, $whole_content )>.
 
 Returns just the content in scalar context.
 
@@ -1489,25 +1780,35 @@ sub run_test {
 
     return (
         $ret->[0],
-        HTTP::Headers->new( @{ $ret->[1] } ),
+        HTTP::Headers::Fast->new( @{ $ret->[1] } ),
         join '', @{ $ret->[2] },
     );
 };
 
-=head2 get_routes
+=head2 $neaf->get_routes( $callback->(\%route_def, $path, $method) )
+
+Returns a 2-level hashref with ALL routes for inspection.
+
+So C<$hash{'/path'}{'GET'} = { handler, expected params, description etc }>
+
+If callback is present, run it against route definition
+and append to hash its return value, but ONLY if it's true.
+
+As of 0.20, route definitions are only protected by shallow copy,
+so be careful with them.
 
-Returns a hash with ALL routes for inspection.
 This should NOT be used by application itself.
 
 =cut
 
 sub get_routes {
-    my $self = shift;
+    my ($self, $code) = @_;
     $self = $Inst unless ref $self;
 
+    $code ||= sub { $_[0] };
+    scalar $self->run; # burn caches
+
     # TODO 0.30 must do deeper copying
-    # TODO 0.30 need callback here
-    # TODO 0.30 filter routes by path & method
     my $all = $self->{route};
     my %ret;
     foreach my $path ( keys %$all ) {
@@ -1516,14 +1817,16 @@ sub get_routes {
             my $route = $batch->{$method};
             $self->_post_setup( $route )
                 unless $route->{lock};
-            $ret{$path}{$method} = { %$route };
+
+            my $filtered = $code->( { %$route }, $path, $method );
+            $ret{$path}{$method} = $filtered if $filtered;
         };
     };
 
     return \%ret;
 };
 
-=head2 set_forced_view( $view )
+=head2 $neaf->set_forced_view( $view )
 
 If set, this view object will be user instead of ANY other view.
 
@@ -1545,7 +1848,7 @@ sub set_forced_view {
     return $self;
 };
 
-=head2 server_stat ( MVC::Neaf::X::ServerStat->new( ... ) )
+=head2 $neaf->server_stat ( MVC::Neaf::X::ServerStat->new( ... ) )
 
 Record server performance statistics during run.
 
@@ -1571,6 +1874,9 @@ or time is exceeded by difference between first and last request in batch.
 
 Returns self.
 
+B<DEPRECATED.> Just use pre_route/pre_reply/pre_cleanup hooks if you need
+to gather performance statistics.
+
 =cut
 
 sub server_stat {
@@ -1596,7 +1902,7 @@ unless you want something very strange.
 =cut
 
 
-=head2 new(%options)
+=head2 MVC::Neaf->new(%options)
 
 Constructor. Usually, instantiating Neaf is not required.
 But it's possible.
@@ -1624,7 +1930,7 @@ sub new {
     return $self;
 };
 
-=head2 handle_request( MVC::Neaf::Request->new )
+=head2 $neaf->handle_request( MVC::Neaf::Request->new )
 
 This is the CORE of this module.
 Should not be called directly - use C<run()> instead.
@@ -1910,51 +2216,56 @@ sub _post_setup {
 sub _error_to_reply {
     my ($self, $req, $err) = @_;
 
-    # TODO 0.30 Neaf::Exception === die 404
-    if (blessed $err and $err->isa("MVC::Neaf::Exception")) {
-        $err->{-status} ||= 500;
-        return $err;
+    # Convert all errors to Neaf expt.
+    if (!blessed $err) {
+        $err = MVC::Neaf::Exception->new(
+            -status   => $err,
+            -nocaller => 1,
+        );
+    }
+    elsif ( !$err->isa("MVC::Neaf::Exception")) {
+        $err = MVC::Neaf::Exception->new(
+            -status   => 500,
+            -sudden   => 1,
+            -reason   => $err,
+            -nocaller => 1,
+        );
     };
 
-    my $status = (!ref $err && $err =~ /^(\d\d\d)\s/) ? $1 : 500;
-    my $sudden = !$1;
+    # Now $err is guaranteed to be a Neaf error
 
-    # Try exception handler
-    if( $sudden and exists $self->{on_error}) {
+    # Use on_error callback to fixup error or gather stats
+    if( $err->is_sudden and exists $self->{on_error}) {
         eval {
             $self->{on_error}->($req, $err, $req->endpoint_origin);
-            $sudden = 0;
             1;
         }
             or $req->log_error( "on_error callback failed: ".($@ || "unknown reason") );
     };
 
     # Try fancy error template
-    if (exists $self->{error_template}{$status}) {
+    if (my $tpl = $self->{error_template}{$err->status}) {
         my $ret = eval {
-            $self->{error_template}{$status}->( $req,
-                status => $status,
-                caller => $req->endpoint_origin,
+            $tpl->( $req,
+                status => $err->status,
+                caller => $req->endpoint_origin, # TODO 0.25 kill this
                 error => $err,
             );
         };
         if (ref $ret eq 'HASH') {
-            $ret->{-status} ||= $status;
+            # success
+            $ret->{-status} ||= $err->status;
             return $ret;
         };
-        $req->log_error( "error_template for $status failed:"
+        $req->log_error( "error_template for ".$err->status." failed:"
             .( $@ || "unknown reason") );
     };
 
-    # Options exhausted - return plain error message
-    $req->log_error( $err )
-        if $sudden;
-    my $req_id = $req->id;
-    return {
-        -status     => $status,
-        -type       => 'application/json',
-        -content    => qq({"error":"$status","req_id":"$req_id"}),
-    };
+    # Options exhausted - return plain error message,
+    #    keep track of reason on the inside
+    $req->log_error( $err->reason )
+        if $err->is_sudden;
+    return $err->make_reply( $req );
 };
 
 sub _croak {
@@ -1965,11 +2276,11 @@ sub _croak {
     croak( (ref $self || $self)."->$where: $msg" );
 };
 
-=head2 get_view( "name" )
+=head2 $neaf->get_view( "name", $lazy )
 
 Fetch view object by name.
 
-Uses C<load_view> ( name => name ) if needed.
+Uses C<load_view> ( name => name ) if needed, unless $lazy flag on.
 
 This is for internal usage.
 
@@ -1978,7 +2289,7 @@ If C<set_forced_view> was called, return its argument instead.
 =cut
 
 sub get_view {
-    my ($self, $view) = @_;
+    my ($self, $view, $lazy) = @_;
     $self = $Inst unless ref $self;
 
     # We've been overridden!
@@ -1991,13 +2302,13 @@ sub get_view {
 
     # Try loading & caching if not present.
     $self->load_view( $view, $view )
-        unless $self->{seen_view}{$view};
+        unless $lazy || $self->{seen_view}{$view};
 
     # Finally, return the thing.
     return $self->{seen_view}{$view};
 };
 
-=head2 get_form( "name" )
+=head2 $neaf->get_form( "name" )
 
 Fetch form named "name". No magic here. See C<add_form>.
 
@@ -2033,6 +2344,7 @@ All of them are supposed to start and end with:
 
     neaf->static( '/images' => "/local/images" );
     neaf->static( '/favicon.ico' => "/local/images/icon_32x32.png" );
+    neaf->static( '/robots.txt' => [ "Disallow: *\n", "text/plain "] );
 
 =head2 Form submission
 
@@ -2195,8 +2507,6 @@ Feedback and/or critique are welcome.
 =head1 SUPPORT
 
 Feel free to email the author to get instant help!
-Or you can comment the L<announce|http://perlmonks.org/?node_id=1174241>
-at the Perlmonks forum.
 
 You can find documentation for this module with the perldoc command.
 
@@ -2234,7 +2544,7 @@ L<http://search.cpan.org/dist/MVC-Neaf/>
 The L<Kelp> framework has very similar concept.
 
 Neaf has a lot of similarities to L<Mojolicious::Lite>,
-all of them unintentional.
+initially unintentional.
 
 =head1 ACKNOWLEDGEMENTS
 
@@ -2257,11 +2567,12 @@ Thanks to L<Cono|https://github.com/cono>
 for early feedback and feature proposals.
 
 Thanks to Alexey Kuznetsov
-for requesting multiple methods.
+for requesting RESTfullness and thus
+adding of multiple methods for the same path.
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2016-2017 Konstantin S. Uvarin L<khedin@cpan.org>.
+Copyright 2016-2017 Konstantin S. Uvarin C<khedin@cpan.org>.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
