@@ -3,7 +3,7 @@ package MVC::Neaf::Request;
 use strict;
 use warnings;
 
-our $VERSION = 0.2002;
+our $VERSION = 0.21;
 
 =head1 NAME
 
@@ -45,7 +45,7 @@ Thus it is expected to have the following methods.
 use Carp;
 use URI::Escape;
 use Encode;
-use HTTP::Headers;
+use HTTP::Headers::Fast;
 use Time::HiRes ();
 use Sys::Hostname ();
 use Digest::MD5 qw(md5_base64);
@@ -239,7 +239,7 @@ sub script_name {
     my $self = shift;
 
     carp "NEAF: script_name call before routing was applied is DEPRECATED"
-        unless $self->{route};
+        unless $self->{endpoint};
 
     return $self->{script_name} ||= $self->path;
 };
@@ -340,57 +340,11 @@ sub path_info_split {
     return @{ $self->{path_info_split} || [] };
 };
 
-=head2 set_full_path( $path )
-
-=head2 set_full_path( $script_name, $path_info )
-
-Set new path elements which will be returned from this point onward.
-
-Also updates path() value so that path = script_name + path_info
-still holds.
-
-set_full_path(undef) resets script_name to whatever returned
-by the underlying driver.
-
-Returns self.
-
-B<DEPRECATED> Use set_path() and set_path_info() instead.
-
-=cut
-
-sub set_full_path {
-    my ($self, $script_name, $path_info) = @_;
-
-    carp "NEAF: set_full_path() is DEPRECATED and will be removed in 0.20";
-
-    if (!defined $script_name) {
-        $script_name = $self->do_get_path;
-    };
-
-    # CANONIZE
-    $script_name =~ s#^/*#/#;
-    $self->{script_name} = $script_name;
-
-    if (defined $path_info) {
-        # Make sure path_info always has a slash if nonempty
-        $path_info =~ s#^/+##;
-        $self->{path_info} = Encode::is_utf8($path_info)
-                ? $path_info
-                : decode_utf8(uri_unescape($path_info));
-    } elsif (!defined $self->{path_info}) {
-        $self->{path_info} = '';
-    };
-    # assert $self->{path_info} is defined by now
-
-    $self->{path} = "$self->{script_name}"
-        .(length $self->{path_info} ? "/$self->{path_info}" : '');
-    return $self;
-};
 
 sub _import_route {
     my ($self, $route, $path, $path_info, $tail) = @_;
 
-    $self->{route}        = $route;
+    $self->{endpoint}        = $route;
     $self->{script_name}  = $path;
     $self->{path_info}    = $path_info;
     $self->{path_info_split}   = $tail;
@@ -460,7 +414,7 @@ See L<MVC::Neaf::X::Form::Wildcard>.
 sub param {
     my ($self, $name, $regex, $default) = @_;
 
-    $regex ||= $self->{route}{param_regex}{$name};
+    $regex ||= $self->{endpoint}{param_regex}{$name};
 
     $self->_croak( "NEAF: param(): a validation regex is REQUIRED" )
         unless defined $regex;
@@ -539,7 +493,7 @@ Please be careful when upgrading.
 sub multi_param {
     my ($self, $name, $regex) = @_;
 
-    $regex ||= $self->{route}{param_regex}{$name};
+    $regex ||= $self->{endpoint}{param_regex}{$name};
     $self->_croak( "validation regex is REQUIRED" )
         unless defined $regex;
 
@@ -609,18 +563,6 @@ sub form {
     };
 };
 
-=head2 get_form_as_hash ( name => qr/.../, name2 => qr/..../, ... )
-
-B<DEPRECATED> and dies. Use L<MVC::Neaf::X::Form> instead.
-
-=cut
-
-sub get_form_as_hash {
-    my ($self, %spec) = @_;
-
-    $self->_croak( "DEPRECATED. use MVC::Neaf::X::Form and form() method" );
-};
-
 =head2 get_form_as_list ( qr/.../, qw(name1 name2 ...)  )
 
 =head2 get_form_as_list ( [ qr/.../, "default" ], qw(name1 name2 ...)  )
@@ -709,7 +651,7 @@ Like above, but no decoding whatsoever is performed.
 
 =head2 upload( "name" )
 
-B<DEPRECATED>. Same as upload_raw.
+B<DEPRECATED>. Same as upload_raw, but issues a warning.
 
 =cut
 
@@ -723,13 +665,6 @@ sub upload_raw {
     return $self->_upload( id => $name, utf8 => 0 );
 };
 
-sub upload {
-    my ($self, $name) = @_;
-
-    carp "NEAF: req->upload() is DEPRECATED, use upload_utf8 or upload_raw instead";
-    return $self->_upload( id => $name, utf8 => 0 );
-};
-
 # TODO 0.30 do something about upload's content type
 # TODO 0.30 and btw, restrict content types!
 sub _upload {
@@ -818,7 +753,8 @@ sub set_cookie {
     defined $opt{regex} and $cook !~ /^$opt{regex}$/
         and $self->_croak( "output value doesn't match regex" );
     if (exists $opt{expires}) {
-        carp( "NEAF set_cookie(): 'expires' parameter detected, use 'expire' instead" );
+        # TODO 0.30 kill it and croak
+        carp( "NEAF set_cookie(): 'expires' parameter DEPRECATED, use 'expire' instead" );
         $opt{expire} = delete $opt{expires};
     };
 
@@ -943,11 +879,11 @@ Fetch HTTP header sent by client.
 Header names are lowercased, dashes converted to underscores.
 So "Http-Header", "HTTP_HEADER" and "http_header" are all the same.
 
-Without argument, returns a L<HTTP::Headers> object.
+Without argument, returns a L<HTTP::Headers::Fast> object.
 
 With a name, returns all values for that header in list context,
 or ", " - joined value as one scalar in scalar context -
-this is actually a frontend to HTTP::Headers header() method.
+this is actually a frontend to HTTP::Headers::Fast header() method.
 
 B<EXPERIMENTAL> The return value format MAY change in the near future.
 
@@ -1186,12 +1122,12 @@ sending confirmation e-mail) may be postponed until the request is served.
 
 =head2 header_out( [$param] )
 
-Without parameters returns a L<HTTP::Headers> object containing all headers
-to be returned to client.
+Without parameters returns a L<HTTP::Headers::Fast>-compatible object
+containing all headers to be returned to client.
 
 With one parameter returns this header.
 
-Returned values are just proxied L<HTTP::Headers> returns.
+Returned values are just proxied L<HTTP::Headers::Fast> returns.
 It is generally advised to use them in list context as multiple
 headers may return trash in scalar context.
 
@@ -1210,7 +1146,7 @@ B<NOTE> This format may change in the future.
 sub header_out {
     my $self = shift;
 
-    my $head = $self->{response}{header} ||= HTTP::Headers->new;
+    my $head = $self->{response}{header} ||= HTTP::Headers::Fast->new;
     return $head unless @_;
 
     my $name = shift;
@@ -1224,7 +1160,7 @@ sub header_out {
 =head2 remove_header( $name )
 
 Set, append, and delete values in the header_out object.
-See L<HTTP::Headers>.
+See L<HTTP::Headers::Fast>.
 
 Arrayrefs are ok and will set multiple values for a given header.
 
@@ -1253,8 +1189,6 @@ This may be useful in postponed actions or hooks.
 
 This is killed by a C<clear()> call.
 
-B<EXPERIMENTAL>. This function MAY be removed or changed in the future.
-
 =cut
 
 sub reply {
@@ -1288,9 +1222,6 @@ Use C<stash> as a last resort for temporary, private data.
 Stash is not killed by C<clear()> function so that cleanup isn't
 botched accidentally.
 
-B<EXPERIMENTAL>. This function MAY be removed if hooks turn out to be
-too cumbersome.
-
 =cut
 
 sub stash {
@@ -1495,8 +1426,8 @@ B<DEPRECATED>. Do not use.
 sub endpoint_origin {
     my $self = shift;
 
-    return '(unspecified file):0' unless $self->{route}{caller};
-    return join ":", @{ $self->{route}{caller} }[1,2];
+    return '(unspecified file):0' unless $self->{endpoint}{caller};
+    return join " line ", @{ $self->{endpoint}{caller} }[1,2];
 };
 
 # If called outside user's code, carp() will point at http server
@@ -1575,7 +1506,7 @@ They shall not generally be called directly inside the app.
 
 =item * do_get_body()
 
-=item * do_get_header_in() - returns a HTTP::Headers object.
+=item * do_get_header_in() - returns a HTTP::Headers::Fast object.
 
 =item * do_reply( $status, $content ) - write reply to client
 
@@ -1633,6 +1564,76 @@ and a corresponding warning being added.
 
 Please keep an eye on C<Changes> though.
 
+Here are these methods, for the sake of completeness.
+
+=head2 set_full_path( $path )
+
+=head2 set_full_path( $script_name, $path_info )
+
+Set new path elements which will be returned from this point onward.
+
+Also updates path() value so that path = script_name + path_info
+still holds.
+
+set_full_path(undef) resets script_name to whatever returned
+by the underlying driver.
+
+Returns self.
+
+B<DEPRECATED> Use set_path() and set_path_info() instead.
+
+=cut
+
+# TODO 0.25 kill it
+sub set_full_path {
+    my ($self, $script_name, $path_info) = @_;
+
+    carp "NEAF: set_full_path() is DEPRECATED and will be removed in 0.25";
+
+    if (!defined $script_name) {
+        $script_name = $self->do_get_path;
+    };
+
+    # CANONIZE
+    $script_name =~ s#^/*#/#;
+    $self->{script_name} = $script_name;
+
+    if (defined $path_info) {
+        # Make sure path_info always has a slash if nonempty
+        $path_info =~ s#^/+##;
+        $self->{path_info} = Encode::is_utf8($path_info)
+                ? $path_info
+                : decode_utf8(uri_unescape($path_info));
+    } elsif (!defined $self->{path_info}) {
+        $self->{path_info} = '';
+    };
+    # assert $self->{path_info} is defined by now
+
+    $self->{path} = "$self->{script_name}"
+        .(length $self->{path_info} ? "/$self->{path_info}" : '');
+    return $self;
+};
+
+# TODO 0.25 kill
+sub upload {
+    my ($self, $name) = @_;
+
+    carp "NEAF: req->upload() is DEPRECATED, use upload_utf8 or upload_raw instead";
+    return $self->_upload( id => $name, utf8 => 0 );
+};
+
+=head2 get_form_as_hash ( name => qr/.../, name2 => qr/..../, ... )
+
+B<DEPRECATED> and dies. Use L<MVC::Neaf::X::Form> instead.
+
+=cut
+
+sub get_form_as_hash {
+    my ($self, %spec) = @_;
+
+    $self->_croak( "DEPRECATED. use neaf form 'foo' => { ... }" );
+};
+
 =head2 set_default( key => $value, ... )
 
 As of v.0.20 this dies.
@@ -1665,7 +1666,7 @@ sub get_default {
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2016-2017 Konstantin S. Uvarin L<khedin@cpan.org>.
+Copyright 2016-2017 Konstantin S. Uvarin C<khedin@cpan.org>.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
