@@ -2,7 +2,7 @@ package MVC::Neaf::X::Session::Base;
 
 use strict;
 use warnings;
-our $VERSION = 0.2002;
+our $VERSION = 0.21;
 
 =head1 DESCRIPTION
 
@@ -23,10 +23,10 @@ Not Even A Framework.
 
 =cut
 
-use parent qw(MVC::Neaf::X::Session);
-use JSON::MaybeXS;
 use Carp;
 
+use MVC::Neaf::Util qw(encode_json decode_json);
+use parent qw(MVC::Neaf::X::Session);
 
 =head2 new( %options )
 
